use std::time::Duration;
use chrono::{DateTime, Local};

pub enum Segment {
    Time(Duration),
    Size(u64),
}

pub fn format_filename(file_name: &str) -> String {
    let local: DateTime<Local> = Local::now();
    // let time_str = local.format("%Y-%m-%dT%H_%M_%S");
    let time_str = local.format(file_name);
    // format!("{file_name}{time_str}")
    time_str.to_string()
}
